@@ -38,13 +38,9 @@ cryfscreateoptions::cryfscreateoptions( QWidget * parent,
 
 	m_ui->lineEdit->setText( "32768" ) ;
 
-	Task::run< utility::Task >( [](){
+	auto exe = utility::executableFullPath( "cryfs" ) + " --show-ciphers" ;
 
-		auto exe = utility::executableFullPath( "cryfs" ) + " --show-ciphers" ;
-
-		return utility::Task( exe,utility::systemEnvironment(),[](){},false ) ;
-
-	} ).then( [ this ]( const utility::Task& e ){
+	utility::Task::run( exe ).then( [ this ]( const utility::Task& e ){
 
 		if( e.success() ){
 
@@ -82,21 +78,20 @@ void cryfscreateoptions::pbOK()
 		s = "32768" ;
 	}
 
-	m_function( e.arg( m_ui->comboBox->currentText(),s ) ) ;
-
-	this->HideUI() ;
+	this->HideUI( e.arg( m_ui->comboBox->currentText(),s ) ) ;
 }
 
 void cryfscreateoptions::pbCancel()
 {
-	m_function( QString() ) ;
-
-	this->HideUI() ;
+	this->HideUI( QString() ) ;
 }
 
-void cryfscreateoptions::HideUI()
+void cryfscreateoptions::HideUI( const QString& e )
 {
 	this->hide() ;
+
+	m_function( e ) ;
+
 	this->deleteLater() ;
 }
 
