@@ -47,7 +47,6 @@
 #include <unistd.h>
 
 #include "filemanager.h"
-#include "keydialog.h"
 #include "dialogmsg.h"
 #include "tablewidget.h"
 #include "oneinstance.h"
@@ -60,24 +59,28 @@
 
 #include "json.h"
 
+static QVector< std::pair< favorites::entry,QByteArray > > _readFavorites()
+{
+	QVector< std::pair< favorites::entry,QByteArray > > e ;
+
+	for( auto&& it : utility::readFavorites() ){
+
+		e.append( { std::move( it ),QByteArray() } ) ;
+	}
+
+	return e ;
+}
+
 sirikali::sirikali() :
 	m_secrets( this ),
 	m_mountInfo( this,true,[ & ](){ QCoreApplication::exit( m_exitStatus ) ; } )
 {
 }
 
-/*
- * Below 3 should be the only function that closes the application
- */
-void sirikali::closeApplication()
-{
-	utility::quitHelper() ;
-	this->hide() ;
-	m_mountInfo.stop()() ;
-}
-
 void sirikali::closeApplication( int s,const QString& e )
 {
+	utility::quitHelper() ;
+
 	m_exitStatus = s ;
 
 	if( !e.isEmpty() ){
@@ -85,25 +88,17 @@ void sirikali::closeApplication( int s,const QString& e )
 		utility::debug() << e ;
 	}
 
-	this->hide() ;
-	m_mountInfo.stop()() ;
-}
+	if( m_ui ){
 
-void sirikali::closeApplication_1( int s )
-{
-	Q_UNUSED( s ) ;
-	m_mountInfo.stop()() ;
-}
-
-void sirikali::setUpApp( bool start,const QString& volume )
-{
-	if( !start ){
-
-		DialogMsg( this ).ShowUIOK( tr( "ERROR" ),tr( "Failed To Start Helper Application.\n\"org.sirikali.pkexec.policy\" polkit file is either misconfigured or sirikali.pkexec executable could not be found." ) ) ;
-
-		return this->closeApplication() ;
+		this->hide() ;
+		utility::Task::suspendForOneSecond() ;
 	}
 
+	m_mountInfo.stop() ;
+}
+
+void sirikali::setUpApp( const QString& volume )
+{
 	this->setLocalizationLanguage( true ) ;
 
 	m_ui = new Ui::sirikali ;
@@ -115,18 +110,16 @@ void sirikali::setUpApp( bool start,const QString& volume )
 	m_ui->pbupdate->setMinimumHeight( 31 ) ;
 	m_ui->pbFavorites->setMinimumHeight( 31 ) ;
 
-	auto f = utility::getWindowDimensions() ;
-
-	auto e = f.data() ;
-
-	this->window()->setGeometry( *( e + 0 ),*( e + 1 ),*( e + 2 ),*( e + 3 ) ) ;
-
 	auto table = m_ui->tableWidget ;
 
-	table->setColumnWidth( 0,*( e + 4 ) ) ;
-	table->setColumnWidth( 1,*( e + 5 ) ) ;
-	table->setColumnWidth( 2,*( e + 6 ) ) ;
-	table->setColumnWidth( 3,*( e + 7 ) ) ;
+	const auto dimensions = utility::getWindowDimensions() ;
+
+	this->window()->setGeometry( dimensions.geometry() ) ;
+
+	table->setColumnWidth( 0,dimensions.columnWidthAt( 0 ) ) ;
+	table->setColumnWidth( 1,dimensions.columnWidthAt( 1 ) ) ;
+	table->setColumnWidth( 2,dimensions.columnWidthAt( 2 ) ) ;
+	table->setColumnWidth( 3,dimensions.columnWidthAt( 3 ) ) ;
 
 #if QT_VERSION < QT_VERSION_CHECK( 5,0,0 )
 	m_ui->tableWidget->verticalHeader()->setResizeMode( QHeaderView::ResizeToContents ) ;
@@ -202,7 +195,7 @@ void sirikali::setUpApp( bool start,const QString& volume )
 
 	this->disableAll() ;
 
-	this->updateVolumeList( siritask::updateVolumeList().await() ) ;
+	this->updateVolumeList( mountinfo::unlockedVolumes().await() ) ;
 
 	if( volume.isEmpty() ) {
 
@@ -318,13 +311,8 @@ void sirikali::setUpAppMenu()
 	m->addAction( _addAction( true,checkForUpdates::autoCheck(),tr( "Autocheck For Updates" ),
 				  "Autocheck For Updates",SLOT( autoCheckUpdates( bool ) ) ) ) ;
 
-	if( utility::platformIsLinux() ){
-
-		m->addAction( _addAction( true,utility::enablePolkitSupport(),
-					  tr( "Enable Polkit Support" ),
-					  "Enable Polkit Support",
-					  SLOT( enablePolkitSupport( bool ) ) ) ) ;
-	}
+	m->addAction( _addAction( true,utility::startMinimized(),tr( "Start Minimized" ),
+				  "Start Minimized",SLOT( startMinimized( bool ) ) ) ) ;
 
 	m->addAction( _addAction( false,false,tr( "Set Mount Point Prefix" ),
 				  "Set Mount Point Prefix",SLOT( setDefaultMountPointPrefix() ) ) ) ;
@@ -415,6 +403,11 @@ void sirikali::setUpAppMenu()
 		 this,SLOT( slotTrayClicked( QSystemTrayIcon::ActivationReason ) ) ) ;
 }
 
+void sirikali::startMinimized( bool e )
+{
+	utility::setStartMinimized( e ) ;
+}
+
 void sirikali::setFileManager()
 {
 	fileManager::instance( this,m_folderOpener,true ) ;
@@ -601,19 +594,13 @@ void sirikali::favoriteClicked( QAction * ac )
 	}else{
 		if( e == tr( "Mount All" ).remove( '&' ) ){
 
-			for( const auto& it : this->autoUnlockVolumes( utility::readFavorites() ) ){
-
-				this->showMoungDialog( it ) ;
-			}
+			this->mountMultipleVolumes( _readFavorites() ) ;
 		}else{
-			for( const auto& it : utility::readFavorites() ){
+			for( auto&& it : _readFavorites() ){
 
-				if( it.volumePath == e ){
+				if( it.first.volumePath == e ){
 
-					if( !this->autoUnlockVolumes( { it } ).isEmpty() ){
-
-						this->showMoungDialog( it ) ;
-					}
+					this->mountMultipleVolumes( this->autoUnlockVolumes( { std::move( it ) },true ) ) ;
 
 					break ;
 				}
@@ -624,8 +611,10 @@ void sirikali::favoriteClicked( QAction * ac )
 
 void sirikali::showFavorites()
 {
-	utility::readFavorites( m_ui->pbFavorites->menu(),true,
-				tr( "Manage Favorites" ),tr( "Mount All" ) ) ;
+	utility::readFavorites( m_ui->pbFavorites->menu(),
+				true,
+				tr( "Manage Favorites" ),
+				tr( "Mount All" ) ) ;
 }
 
 void sirikali::setLocalizationLanguage( bool translate )
@@ -661,13 +650,6 @@ void sirikali::reuseMountPoint( bool e )
 	utility::reUseMountPoint( e ) ;
 }
 
-void sirikali::enablePolkitSupport( bool e )
-{
-	utility::enablePolkitSupport( e ) ;
-
-	DialogMsg( this ).ShowUIOK( tr( "Warning" ),tr( "A Restart Is Required For The Change To Take Effect." ) );
-}
-
 void sirikali::autoMountFavoritesOnStartUp( bool e )
 {
 	utility::autoMountFavoritesOnStartUp( e ) ;
@@ -721,6 +703,12 @@ int sirikali::start( QApplication& e )
 void sirikali::start( const QStringList& l )
 {
 	m_startHidden  = l.contains( "-e" ) ;
+
+	if( !m_startHidden ){
+
+		m_startHidden = utility::startMinimized() ;
+	}
+
 	m_folderOpener = utility::cmdArgumentValue( l,"-m",utility::fileManager() ) ;
 
 	utility::createFolder( utility::homeConfigPath() ) ;
@@ -741,22 +729,23 @@ void sirikali::start( const QStringList& l )
 
 		utility::createFolder( s ) ;
 
-		if( !utility::pathIsWritable( s ) ){
+		if( utility::pathIsWritable( s ) ){
 
+			oneinstance::callbacks cb = {
+
+				[ this ]( const QString& e ){ this->setUpApp( e ) ; },
+				[ this ](){ this->closeApplication( 1 ) ; },
+				[ this ]( const QString& e ){ this->raiseWindow( e ) ; },
+			} ;
+
+			auto x = utility::cmdArgumentValue( l,"-d" ) ;
+
+			oneinstance::instance( this,s + "/SiriKali.socket",x,std::move( cb ) ) ;
+		}else{
 			DialogMsg( this ).ShowUIOK( tr( "ERROR" ),tr( "\"%1\" Folder Must Be Writable" ).arg( s ) ) ;
 
-			return this->closeApplication() ;
+			this->closeApplication() ;
 		}
-
-		oneinstance::instance( this,
-				       s + "/SiriKali.socket",
-				       utility::cmdArgumentValue( l,"-d" ),
-				       [ this ]( const QString& e ){ utility::startHelperExecutable( this,
-												     e,
-												     "setUpApp",
-												     "closeApplication" ) ; },
-				       [ this ]( int s ){ this->closeApplication_1( s ) ; },
-				       [ this ]( const QString& e ){ this->raiseWindow( e ) ; } ) ;
 	}
 }
 
@@ -778,9 +767,11 @@ void sirikali::cliCommand( const QStringList& l )
 
 	if( l.contains( "-u" ) ){
 
+		m_mountInfo.announceEvents( false ) ;
+
 		auto volume = utility::cmdArgumentValue( l,"-d" ) ;
 
-		for( const auto& it : siritask::updateVolumeList().await() ){
+		for( const auto& it : mountinfo::unlockedVolumes().await() ){
 
 			const auto& a = it.volumePath() ;
 			const auto& b = it.mountPoint() ;
@@ -804,7 +795,7 @@ void sirikali::cliCommand( const QStringList& l )
 
 	if( l.contains( "-p" ) ){
 
-		for( const auto& it : siritask::updateVolumeList().await() ){
+		for( const auto& it : mountinfo::unlockedVolumes().await() ){
 
 			it.printVolumeInfo() ;
 		}
@@ -848,13 +839,15 @@ void sirikali::unlockVolume( const QStringList& l )
 				}
 			}() ;
 
+			m_mountInfo.announceEvents( false ) ;
+
 			siritask::options s = { volume,m,key,idleTime,cPath,QString(),mode,mOpt,QString() } ;
 
 			auto& e = siritask::encryptedFolderMount( s ) ;
 
 			if( e.await() == siritask::status::success ){
 
-				this->openMountPointPath( m ) ;
+				//this->openMountPointPath( m ) ;
 
 				this->closeApplication( 0 ) ;
 			}else{
@@ -879,11 +872,7 @@ void sirikali::unlockVolume( const QStringList& l )
 
 		if( backEnd == "keyfile" ){
 
-			QFile f( keyFile ) ;
-
-			f.open( QIODevice::ReadOnly ) ;
-
-			auto key = f.readAll() ;
+			auto key = utility::fileContents( keyFile ) ;
 
 			if( utility::containsAtleastOne( key,'\n','\0','\r' ) ){
 
@@ -945,43 +934,59 @@ void sirikali::unlockVolume( const QStringList& l )
 	}
 }
 
+void sirikali::mountMultipleVolumes( QVector< std::pair< favorites::entry,QByteArray > > e )
+{
+	if( !e.isEmpty() ){
+
+		m_disableEnableAll = true ;
+
+		this->disableAll() ;
+
+		keyDialog::instance( this,
+				     m_secrets,
+				     m_autoOpenFolderOnMount,
+				     m_folderOpener,
+				     std::move( e ),
+				     [ this ](){ m_disableEnableAll = false ; this->enableAll() ; } ) ;
+	}
+}
+
 void sirikali::autoMountFavoritesOnAvailable( QString m )
 {
 	if( utility::autoMountFavoritesOnAvailable() ){
 
-		QVector< favorites::entry > e ;
+		QVector< std::pair< favorites::entry,QByteArray > > e ;
 
-		for( const auto& it : utility::readFavorites() ){
+		for( auto&& it : _readFavorites() ){
 
-			if( it.volumePath.startsWith( m ) && it.autoMount() ){
+			if( it.first.volumePath.startsWith( m ) && it.first.autoMount() ){
 
-				e.append( it ) ;
+				e.append( std::move( it ) ) ;
 			}
 		}
 
-		for( const auto& it : this->autoUnlockVolumes( e ) ){
-
-			this->mount( it ) ;
-		}
+		this->mountMultipleVolumes( this->autoUnlockVolumes( std::move( e ) ) ) ;
 	}
 }
 
 void sirikali::autoUnlockVolumes()
 {
-	QVector< favorites::entry > e ;
+	QVector< std::pair< favorites::entry,QByteArray > > e ;
 
-	for( const auto& it : utility::readFavorites() ){
+	for( auto&& it : _readFavorites() ){
 
-		if( it.autoMount() ){
+		if( it.first.autoMount() ){
 
-			e.append( it ) ;
+			e.append( std::move( it ) ) ;
 		}
 	}
 
-	this->autoUnlockVolumes( e ) ;
+	this->mountMultipleVolumes( this->autoUnlockVolumes( std::move( e ) ) ) ;
 }
 
-QVector< favorites::entry > sirikali::autoUnlockVolumes( const QVector< favorites::entry >& l )
+QVector< std::pair< favorites::entry,QByteArray > >
+sirikali::autoUnlockVolumes( const QVector< std::pair< favorites::entry,QByteArray > > l,
+							 bool autoOpenFolderOnMount )
 {
 	if( l.isEmpty() ){
 
@@ -1004,15 +1009,25 @@ QVector< favorites::entry > sirikali::autoUnlockVolumes( const QVector< favorite
 
 	auto _mountVolumes = [ & ](){
 
-		QVector< favorites::entry > e ;
-
-		auto _mount = [ this ]( const favorites::entry& e,const QByteArray& key,bool s ){
+		QVector< std::pair< favorites::entry,QByteArray > > e ;
 
+		auto _mount = [ & ]( QVector< std::pair< favorites::entry,QByteArray > >& q,
+				     const std::pair< favorites::entry,QByteArray >& e,
+				     const QByteArray& key,
+				     bool s ){
 			if( s ){
 
-				this->mount( e,QString(),key ) ;
+				q.append( { e.first,key } ) ;
 			}else{
-				siritask::encryptedFolderMount( { e,key } ).start() ;
+				auto& s = siritask::encryptedFolderMount( { e.first,key } ) ;
+
+				s.then( [ this,autoOpenFolderOnMount,e = e.first.mountPointPath ]( siritask::cmdStatus s ){
+
+					if( s == siritask::status::success && autoOpenFolderOnMount ){
+
+						this->openMountPointPath( e ) ;
+					}
+				} ) ;
 			}
 		} ;
 
@@ -1020,13 +1035,13 @@ QVector< favorites::entry > sirikali::autoUnlockVolumes( const QVector< favorite
 
 		for( const auto& it : l ){
 
-			const auto key = m->readValue( it.volumePath ) ;
+			const auto key = m->readValue( it.first.volumePath ) ;
 
 			if( key.isEmpty() ){
 
 				e.append( it ) ;
 			}else{
-				_mount( it,key,s ) ;
+				_mount( e,it,key,s ) ;
 			}
 		}
 
@@ -1067,7 +1082,7 @@ void sirikali::ecryptfsProperties()
 		}
 	}() ;
 
-	for( const auto& it : siritask::updateVolumeList().await() ){
+	for( const auto& it : mountinfo::unlockedVolumes().await() ){
 
 		if( it.mountPoint() == s ){
 
@@ -1080,7 +1095,7 @@ void sirikali::ecryptfsProperties()
 				s.replace( "rw\n\n","mode=read and write\n\n" ) ;
 				s.replace( "="," = " ) ;
 
-				return s;
+				return s ;
 			}() ) ;
 
 			break ;
@@ -1298,7 +1313,7 @@ void sirikali::showContextMenu( QTableWidgetItem * item,bool itemClicked )
 
 	m.setFont( this->font() ) ;
 
-	auto _addAction = [ & ]( const QString& txt,const volumeType& e ){
+	auto _addAction = [ & ]( const auto& txt,const volumeType& e ){
 
 		auto ac = m.addAction( txt ) ;
 
@@ -1426,7 +1441,7 @@ void sirikali::hideWindow()
 
 void sirikali::setUpShortCuts()
 {
-	auto _addAction = [ this ]( std::initializer_list<QKeySequence> s,const char * slot ){
+	auto _addAction = [ this ]( std::initializer_list< QKeySequence > s,const char * slot ){
 
 		auto ac = new QAction( this ) ;
 
@@ -1457,8 +1472,6 @@ void sirikali::setUpShortCuts()
 
 		return e ;
 	}() ) ;
-
-
 }
 
 void sirikali::setUpFont()
@@ -1491,25 +1504,28 @@ void sirikali::dragEnterEvent( QDragEnterEvent * e )
 
 void sirikali::dropEvent( QDropEvent * e )
 {
+	QVector< std::pair< favorites::entry,QByteArray > > s ;
+
 	for( const auto& it : e->mimeData()->urls() ){
 
-		this->showMoungDialog( it.path() ) ;
+		s.append( { it.path(),QByteArray() } ) ;
 	}
+
+	this->mountMultipleVolumes( std::move( s ) ) ;
 }
 
 void sirikali::mount( const volumeInfo& entry,const QString& exe,const QByteArray& key )
 {
 	this->disableAll() ;
 
-	keyDialog::instance( this,m_ui->tableWidget,m_secrets,entry,[ this ](){
-
-		this->enableAll() ;
-
-	},[ this ]( const QString& e ){
-
-		this->openMountPointPath( e ) ;
-
-	},exe,key ) ;
+	keyDialog::instance( this,
+			     m_secrets,
+			     entry,
+			     [ this ](){ this->enableAll() ; },
+			     m_autoOpenFolderOnMount,
+			     m_folderOpener,
+			     exe,
+			     key ) ;
 }
 
 void sirikali::createVolume( QAction * ac )
@@ -1648,6 +1664,8 @@ void sirikali::pbUmount()
 		auto b = table->item( row,1 )->text() ;
 		auto c = table->item( row,2 )->text() ;
 
+		utility::Task::suspendForOneSecond() ;
+
 		if( siritask::encryptedFolderUnMount( a,b,c ).await() ){
 
 			siritask::deleteMountFolder( b ) ;
@@ -1667,33 +1685,26 @@ void sirikali::unMountAll()
 
 	auto table = m_ui->tableWidget ;
 
-	auto cipherFolders = tablewidget::columnEntries( table,0 ) ;
-	auto mountPoints   = tablewidget::columnEntries( table,1 ) ;
-	auto fileSystems   = tablewidget::columnEntries( table,2 ) ;
+	const auto cipherFolders = tablewidget::columnEntries( table,0 ) ;
+	const auto mountPoints   = tablewidget::columnEntries( table,1 ) ;
+	const auto fileSystems   = tablewidget::columnEntries( table,2 ) ;
 
-	int r = cipherFolders.size() - 1 ;
+	utility::Task::suspendForOneSecond() ;
 
-	if( r < 0 ){
+	for( auto r = cipherFolders.size() - 1 ; r >= 0 ; r-- ){
 
-		utility::Task::suspendForOneSecond() ;
-	}else{
-		do{
-			const auto& a = cipherFolders.at( r ) ;
-			const auto& b = mountPoints.at( r ) ;
-			const auto& c = fileSystems.at( r ) ;
+		const auto& a = cipherFolders.at( r ) ;
+		const auto& b = mountPoints.at( r ) ;
+		const auto& c = fileSystems.at( r ) ;
 
-			if( siritask::encryptedFolderUnMount( a,b,c ).await() ){
+		if( siritask::encryptedFolderUnMount( a,b,c ).await() ){
 
-				tablewidget::deleteRow( table,b,1 ) ;
+			tablewidget::deleteRow( table,b,1 ) ;
 
-				siritask::deleteMountFolder( b ) ;
+			siritask::deleteMountFolder( b ) ;
 
-				utility::Task::suspendForOneSecond() ;
-			}
-
-			r -= 1 ;
-
-		}while( r >= 0 ) ;
+			utility::Task::suspendForOneSecond() ;
+		}
 	}
 
 	this->enableAll() ;
@@ -1715,7 +1726,7 @@ void sirikali::pbUpdate()
 {
 	this->disableAll() ;
 
-	this->updateVolumeList( siritask::updateVolumeList().await() ) ;
+	this->updateVolumeList( mountinfo::unlockedVolumes().await() ) ;
 }
 
 void sirikali::updateVolumeList( const QVector< volumeInfo >& r )
@@ -1747,7 +1758,7 @@ void sirikali::disableAll()
 
 void sirikali::enableAll()
 {
-	if( !m_removeAllVolumes ){
+	if( !m_disableEnableAll ){
 
 		m_ui->pbmenu->setEnabled( true ) ;
 		m_ui->pbupdate->setEnabled( true ) ;
@@ -1759,12 +1770,6 @@ void sirikali::enableAll()
 	}
 }
 
-void sirikali::enableAll_1()
-{
-	m_removeAllVolumes = false ;
-	this->enableAll() ;
-}
-
 sirikali::~sirikali()
 {
 	if( m_ui ){
@@ -1773,14 +1778,14 @@ sirikali::~sirikali()
 
 		const auto& r = this->window()->geometry() ;
 
-		utility::setWindowDimensions( { r.x(),
-						r.y(),
-						r.width(),
-						r.height(),
-						q->columnWidth( 0 ),
-						q->columnWidth( 1 ),
-						q->columnWidth( 2 ),
-						q->columnWidth( 3 ) } ) ;
+		utility::setWindowDimensions( { { { r.x(),
+						    r.y(),
+						    r.width(),
+						    r.height(),
+						    q->columnWidth( 0 ),
+						    q->columnWidth( 1 ),
+						    q->columnWidth( 2 ),
+						    q->columnWidth( 3 ) } } } ) ;
 
 		delete m_ui ;
 	}
