@@ -51,7 +51,7 @@ static bool _ecryptfs( const T& e )
 
 static bool _ecryptfs_illegal_path( const siritask::options& opts )
 {
-	if( _ecryptfs( opts.type ) && utility::useZuluPolkit() ){
+	if( _ecryptfs( opts.type ) && utility::useSiriPolkit() ){
 
 		return opts.cipherFolder.contains( " " ) || opts.plainFolder.contains( " " ) ;
 	}else{
@@ -121,15 +121,17 @@ Task::future< bool >& siritask::encryptedFolderUnMount( const QString& cipherFol
 {
 	return Task::run< bool >( [ = ](){
 
+		auto ecryptfs = _ecryptfs( fileSystem ) ;
+
 		auto cmd = [ & ](){
 
-			if( _ecryptfs( fileSystem ) ){
+			if( ecryptfs ){
 
 				auto exe = utility::executableFullPath( "ecryptfs-simple" ) ;
 
 				auto s = exe + " -k " + _makePath( cipherFolder ) ;
 
-				if( utility::runningInMixedMode() ){
+				if( utility::useSiriPolkit() ){
 
 					return _wrap_su( s ) ;
 				}else{
@@ -143,20 +145,50 @@ Task::future< bool >& siritask::encryptedFolderUnMount( const QString& cipherFol
 					return "fusermount -u " + _makePath( mountPoint ) ;
 				}
 			}
-		}() ;
+		} ;
 
-		if( utility::platformIsLinux() ){
+		const int max_count = 5 ;
 
-			utility::Task::waitForOneSecond() ;
-		}
+		if( ecryptfs ){
 
-		for( int i = 0 ; i < 5 ; i++ ){
+			bool not_set = true ;
 
-			if( utility::Task::run( cmd,10000,_ecryptfs( fileSystem ) ).get().success() ){
+			auto exe = cmd() ;
 
-				return true ;
-			}else{
-				utility::Task::waitForOneSecond() ;
+			for( int i = 0 ; i < max_count ; i++ ){
+
+				auto s = utility::Task::run( exe,10000,true ).get() ;
+
+				if( s.success() ){
+
+					return true ;
+				}else{
+					if( not_set && s.stdError().contains( "error: failed to set gid" ) ){
+
+						if( utility::enablePolkit( utility::background_thread::True ) ){
+
+							not_set = false ;
+
+							exe = cmd() ;
+						}else{
+							return false ;
+						}
+					}else{
+						utility::Task::waitForOneSecond() ;
+					}
+				}
+			}
+		}else{
+			for( int i = 0 ; i < max_count ; i++ ){
+
+				auto s = utility::Task::run( cmd(),10000,false ).get() ;
+
+				if( s.success() ){
+
+					return true ;
+				}else{
+					utility::Task::waitForOneSecond() ;
+				}
 			}
 		}
 
@@ -241,16 +273,17 @@ static QString _args( const QString& exe,const siritask::options& opt,
 
 				if( create ){
 
-					auto e = QString( "%1 --init %2 %3" ) ;
+					auto e = QString( "%1 --init -q %2 %3" ) ;
 					return e.arg( exe,configPath,cipherFolder ) ;
 				}else{
+					mode += " -o fsname=gocryptfs@" + cipherFolder ;
 
-					if( utility::platformIsOSX() ){
+					//if( utility::platformIsOSX() ){
 
-						mode += " -o fsname=gocryptfs@" + cipherFolder ;
-					}
+					//	mode += " -o volname=" + utility::split( opt.plainFolder,'/' ).last() ;
+					//}
 
-					auto e = QString( "%1 %2 %3 %4 %5" ) ;
+					auto e = QString( "%1 -q %2 %3 %4 %5" ) ;
 					return e.arg( exe,mode,configPath,cipherFolder,mountPoint ) ;
 				}
 			}else{
@@ -259,6 +292,11 @@ static QString _args( const QString& exe,const siritask::options& opt,
 					auto e = QString( "%1 create %2 %3 %4" ) ;
 					return e.arg( exe,opt.createOptions,configPath,cipherFolder ) ;
 				}else{
+					//if( utility::platformIsOSX() ){
+
+					//	mode += " -o volname=" + utility::split( opt.plainFolder,'/' ).last() ;
+					//}
+
 					auto e = QString( "%1 mount -b %2 %3 -o fsname=securefs@%4 -o subtype=securefs %5 %6" ) ;
 					return e.arg( exe,configPath,mode,cipherFolder,cipherFolder,mountPoint ) ;
 				}
@@ -315,14 +353,14 @@ static QString _args( const QString& exe,const siritask::options& opt,
 
 		if( opt.mountOptions.isEmpty() ){
 
-			if( utility::runningInMixedMode() ){
+			if( utility::useSiriPolkit() ){
 
 				return _wrap_su( s ) ;
 			}else{
 				return s ;
 			}
 		}else{
-			if( utility::runningInMixedMode() ){
+			if( utility::useSiriPolkit() ){
 
 				return _wrap_su( s + " -o " + opt.mountOptions ) ;
 			}else{
@@ -334,6 +372,11 @@ static QString _args( const QString& exe,const siritask::options& opt,
 
 		auto e = QString( "%1 %2 %3 %4 %5 %6 -o fsname=%7@%8 -o subtype=%9" ) ;
 
+		//if( utility::platformIsOSX() ){
+
+		//	e += " -o volname=" + utility::split( opt.plainFolder,'/' ).last() ;
+		//}
+
 		auto z = [ & ]()->QString{
 
 			if( create ){
@@ -366,6 +409,11 @@ static QString _args( const QString& exe,const siritask::options& opt,
 	}else{
 		auto e = QString( "%1 %2 %3 %4 %5 %6 -o fsname=%7@%8 -o subtype=%9" ) ;
 
+		if( utility::platformIsOSX() ){
+
+			e += " -o volname=" + utility::split( opt.plainFolder,'/' ).last() ;
+		}
+
 		auto opts = e.arg( exe,cipherFolder,mountPoint,mountOptions,configPath,
 				   separator,type.name(),cipherFolder,type.name() ) ;
 
@@ -388,9 +436,10 @@ static QString _args( const QString& exe,const siritask::options& opt,
 	}
 }
 
-static siritask::status _status( const siritask::volumeType& app,bool s )
+enum class status_type{ exeName,exeNotFound } ;
+static siritask::status _status( const siritask::volumeType& app,status_type s )
 {
-	if( s ){
+	if( s == status_type::exeNotFound ){
 
 		if( app == "cryfs" ){
 
@@ -441,7 +490,7 @@ static siritask::cmdStatus _status( const utility::Task& r,siritask::status s,bo
 
 	siritask::cmdStatus e = { r.exitCode(),stdOut ? r.stdOut() : r.stdError() } ;
 
-	auto msg = e.msg().toLower() ;
+	const auto msg = e.msg().toLower() ;
 
 	/*
 	 *
@@ -455,32 +504,32 @@ static siritask::cmdStatus _status( const utility::Task& r,siritask::status s,bo
 
 		if( msg.contains( "operation not permitted" ) ){
 
-			e.setStatus( siritask::status::ecrypfsBadExePermissions ) ;
+			e = siritask::status::ecrypfsBadExePermissions ;
 
 		}else if( msg.contains( "error: mount failed" ) ){
 
-			e.setStatus( s ) ;
+			e = s ;
 		}
 
 	}else if( s == siritask::status::cryfs ){
 
-		auto& m = e.msg() ;
+		const auto& m = e.msg() ;
 
 		if( msg.contains( "password" ) ){
 
-			e.setStatus( s ) ;
+			e = s ;
 
 		}else if( m.contains( "This filesystem is for CryFS" ) &&
 			  m.contains( "It has to be migrated" ) ){
 
-			e.setStatus( siritask::status::cryfsMigrateFileSystem ) ;
+			e = siritask::status::cryfsMigrateFileSystem ;
 		}
 
 	}else if( s == siritask::status::encfs ){
 
 		if( msg.contains( "password" ) ){
 
-			e.setStatus( s ) ;
+			e = s ;
 		}
 
 	}else if( s == siritask::status::gocryptfs ){
@@ -490,11 +539,11 @@ static siritask::cmdStatus _status( const utility::Task& r,siritask::status s,bo
 		 */
 		if( e.exitCode() == 12 ){
 
-			e.setStatus( s ) ;
+			e = s ;
 		}else{
 			if( msg.contains( "password" ) ){
 
-				e.setStatus( s ) ;
+				e = s ;
 			}
 		}
 
@@ -502,7 +551,7 @@ static siritask::cmdStatus _status( const utility::Task& r,siritask::status s,bo
 
 		if( msg.contains( "password" ) ){
 
-			e.setStatus( s ) ;
+			e = s ;
 		}
 	}
 
@@ -518,16 +567,29 @@ static siritask::cmdStatus _cmd( bool create,const siritask::options& opt,
 
 	if( exe.isEmpty() ){
 
-		return _status( app,true ) ;
+		return _status( app,status_type::exeNotFound ) ;
 	}else{
-		auto e = utility::Task( _args( exe,opt,configFilePath,create ),
-					20000,
-					utility::systemEnvironment(),
-					password.toLatin1(),
-					[](){},
-					_ecryptfs( app ) ) ;
+		auto _run = [ & ](){
 
-		auto s = _status( e,_status( app,false ),app == "encfs" ) ;
+			auto s = utility::Task( _args( exe,opt,configFilePath,create ),
+						20000,
+						utility::systemEnvironment(),
+						password.toLatin1(),
+						[](){},
+						_ecryptfs( app ) ) ;
+
+			return _status( s,_status( app,status_type::exeName ),app == "encfs" ) ;
+		} ;
+
+		auto s = _run() ;
+
+		if( s == siritask::status::ecrypfsBadExePermissions ){
+
+			if( utility::enablePolkit( utility::background_thread::True ) ){
+
+				s = _run() ;
+			}
+		}
 
 		if( s != siritask::status::success ){
 
@@ -701,119 +763,3 @@ Task::future< siritask::cmdStatus >& siritask::encryptedFolderCreate( const siri
 		}
 	} ) ;
 }
-
-Task::future< QVector< volumeInfo > >& siritask::updateVolumeList()
-{
-	return Task::run< QVector< volumeInfo > >( [](){
-
-		auto _hash = []( const QString& e ){
-
-			/*
-			 * jenkins one at a time hash function.
-			 *
-			 * https://en.wikipedia.org/wiki/Jenkins_hash_function
-			 */
-
-			uint32_t hash = 0 ;
-
-			auto p = e.toLatin1() ;
-
-			auto key = p.constData() ;
-
-			auto l = p.size() ;
-
-			for( decltype( l ) i = 0 ; i < l ; i++ ){
-
-				hash += *( key + i ) ;
-
-				hash += ( hash << 10 ) ;
-
-				hash ^= ( hash >> 6 ) ;
-			}
-
-			hash += ( hash << 3 ) ;
-
-			hash ^= ( hash >> 11 ) ;
-
-			hash += ( hash << 15 ) ;
-
-			return QString::number( hash ) ;
-		} ;
-
-		auto _decode = []( QString path,bool set_offset ){
-
-			path.replace( "\\012","\n" ) ;
-			path.replace( "\\040"," " ) ;
-			path.replace( "\\134","\\" ) ;
-			path.replace( "\\011","\\t" ) ;
-
-			if( set_offset ){
-
-				return path.mid( path.indexOf( '@' ) + 1 ) ;
-			}else{
-				return path ;
-			}
-		} ;
-
-		auto _fs = []( QString e ){
-
-			e.replace( "fuse.","" ) ;
-
-			return e ;
-		} ;
-
-		auto _ro = []( const QStringList& l ){
-
-			return l.at( 5 ).mid( 0,2 ) ;
-		} ;
-
-		QVector< volumeInfo > e ;
-
-		volumeInfo::mountinfo info ;
-
-		for( const auto& it : mountinfo().mountedVolumes() ){
-
-			if( volumeInfo::supported( it ) ){
-
-				const auto& k = utility::split( it,' ' ) ;
-
-				const auto s = k.size() ;
-
-				if( s < 6 ){
-
-					continue ;
-				}
-
-				const auto& cf = k.at( s - 2 ) ;
-
-				const auto& m = k.at( 4 ) ;
-
-				const auto& fs = k.at( s - 3 ) ;
-
-				if( utility::startsWithAtLeastOne( cf,"encfs@",
-								   "cryfs@",
-								   "securefs@",
-								   "gocryptfs@" ) ){
-
-					info.volumePath = _decode( cf,true ) ;
-
-				}else if( utility::equalsAtleastOne( fs,"fuse.gocryptfs",
-								     "ecryptfs" ) ){
-
-					info.volumePath = _decode( cf,false ) ;
-				}else{
-					info.volumePath = _hash( m ) ;
-				}
-
-				info.mountPoint   = _decode( m,false ) ;
-				info.fileSystem   = _fs( fs ) ;
-				info.mode         = _ro( k ) ;
-				info.mountOptions = k.last() ;
-
-				e.append( info ) ;
-			}
-		}
-
-		return e ;
-	} ) ;
-}
