@@ -116,6 +116,8 @@ static QSettings * _settings ;
 
 static QByteArray _cookie ;
 
+static bool _use_polkit = false ;
+
 ::Task::future< utility::Task >& utility::Task::run( const QString& exe,bool e )
 {
 	return utility::Task::run( exe,-1,e ) ;
@@ -125,35 +127,28 @@ static QByteArray _cookie ;
 {
 	return ::Task::run< utility::Task >( [ = ](){
 
-		auto env = QProcessEnvironment::systemEnvironment() ;
+		auto env = utility::systemEnvironment() ;
 
-		return utility::Task( exe,s,env,_cookie,[](){ umask( 0 ) ; },e ) ;
+		return utility::Task( exe,s,env,QByteArray(),[](){},e ) ;
 	} ) ;
 }
 
 void utility::Task::execute( const QString& exe,int waitTime,
 			     const QProcessEnvironment& env,
 			     const QByteArray& password,
-			     const std::function< void() >& f,
+			     const std::function< void() >& function,
 			     bool polkit )
 {
-	class Process : public QProcess{
-	public:
-		Process( const std::function< void() >& f ) : m_function( f )
-		{
-		}
-	protected:
-		void setupChildProcess()
-		{
-			m_function() ;
-		}
-	private:
-		std::function< void() > m_function ;
-	} p( f ) ;
+	if( polkit && utility::useSiriPolkit() ){
 
-	p.setProcessEnvironment( env ) ;
+		auto _report_error = [ this ]( const char * msg ){
 
-	if( polkit && utility::useZuluPolkit() ){
+			m_finished   = true ;
+			m_exitCode   = -1 ;
+			m_exitStatus = -1 ;
+			m_stdError   =  msg ;
+			//m_stdOut     =  "" ;
+		} ;
 
 		QLocalSocket s ;
 
@@ -167,6 +162,8 @@ void utility::Task::execute( const QString& exe,int waitTime,
 
 			}else if( i == 3 ){
 
+				_report_error( "SiriKali: Failed To Connect To Polkit Backend" ) ;
+
 				utility::debug() << "ERROR: Failed To Start Helper Application" ;
 				return ;
 			}else{
@@ -180,9 +177,9 @@ void utility::Task::execute( const QString& exe,int waitTime,
 
 			nlohmann::json json ;
 
-			json[ "cookie" ]     = _cookie.constData() ;
-			json[ "password" ]   = password.constData() ;
-			json[ "command" ]    = exe.toLatin1().constData() ;
+			json[ "cookie" ]   = _cookie.constData() ;
+			json[ "password" ] = password.constData() ;
+			json[ "command" ]  = exe.toLatin1().constData() ;
 
 			return json.dump().c_str() ;
 		}() ) ;
@@ -200,8 +197,28 @@ void utility::Task::execute( const QString& exe,int waitTime,
 			m_stdError   = json[ "stdError" ].get< std::string >().c_str() ;
 			m_stdOut     = json[ "stdOut" ].get< std::string >().c_str() ;
 
-		}catch( ... ){}
+		}catch( ... ){
+
+			_report_error( "SiriKali: Failed To Parse Polkit Backend Output" ) ;
+		}
 	}else{
+		class Process : public QProcess{
+		public:
+			Process( const std::function< void() >& function,
+				 const QProcessEnvironment& env ) :
+				m_function( function )
+			{
+				this->setProcessEnvironment( env ) ;
+			}
+		protected:
+			void setupChildProcess()
+			{
+				m_function() ;
+			}
+		private:
+			const std::function< void() >& m_function ;
+		} p( function,env ) ;
+
 		p.start( exe ) ;
 
 		if( !password.isEmpty() ){
@@ -221,40 +238,71 @@ void utility::Task::execute( const QString& exe,int waitTime,
 	}
 }
 
-void utility::startHelperExecutable( QWidget * obj,const QString& arg,const char * slot,const char * slot1 )
+static QString siriPolkitExe()
 {
-	if( !utility::useZuluPolkit() ){
+	auto exe = utility::executableFullPath( "pkexec" ) ;
 
-		QMetaObject::invokeMethod( obj,slot,Q_ARG( bool,true ),Q_ARG( QString,QString() ) ) ;
+	if( exe.isEmpty() ){
 
-		return ;
+		return QString() ;
+	}else{
+		return QString( "%1 %2 %3 fork" ).arg( exe,siriPolkitPath,utility::helperSocketPath() ) ;
+	}
+}
+
+static ::Task::future< utility::Task >& _start_siripolkit( const QString& e )
+{
+	_cookie = plugins::getRandomData( 16 ).toHex() ;
+
+	return ::Task::run< utility::Task >( [ = ]{
+
+		return utility::Task( e,
+				      -1,
+				      utility::systemEnvironment(),
+				      _cookie,
+				      [](){ umask( 0 ) ; },
+				      false ) ;
+	} ) ;
+}
+
+bool utility::enablePolkit( utility::background_thread thread )
+{
+	if( _use_polkit ){
+
+		return true ;
 	}
 
-	QFile f( "/dev/urandom" ) ;
-
-	f.open( QIODevice::ReadOnly ) ;
-
-	_cookie = f.read( 16 ).toHex() ;
-
-	auto exe = utility::executableFullPath( "pkexec" ) ;
+	auto exe = siriPolkitExe() ;
 
 	if( !exe.isEmpty() ){
 
-		exe = QString( "%1 %2 %3 fork" ).arg( exe,siriPolkitPath,utility::helperSocketPath() ) ;
+		auto socketPath = utility::helperSocketPath() ;
 
-		::Task::exec( [ = ](){
+		if( thread == utility::background_thread::True ){
 
-			auto e = utility::Task::run( exe,false ).get() ;
+			if( _start_siripolkit( exe ).get().success() ){
 
-			QMetaObject::invokeMethod( obj,
-						   slot,
-						   Q_ARG( bool,e.success() ),
-						   Q_ARG( QString,arg ) ) ;
-		} ) ;
-	}else{
-		DialogMsg( obj ).ShowUIOK( QObject::tr( "ERROR" ),QObject::tr( "Failed to locate pkexec executable" ) ) ;
-		QMetaObject::invokeMethod( obj,slot1 ) ;
+				_use_polkit = true ;
+
+				while( !utility::pathExists( socketPath ) ){
+
+					utility::Task::waitForOneSecond() ;
+				}
+			}
+		}else{
+			if( _start_siripolkit( exe ).await().success() ){
+
+				_use_polkit = true ;
+
+				while( !utility::pathExists( socketPath ) ){
+
+					utility::Task::suspendForOneSecond() ;
+				}
+			}
+		}
 	}
+
+	return _use_polkit ;
 }
 
 QString utility::helperSocketPath()
@@ -262,26 +310,9 @@ QString utility::helperSocketPath()
 	return utility::homeConfigPath() + ".tmp/SiriKali.polkit.socket" ;
 }
 
-bool utility::useZuluPolkit()
+bool utility::useSiriPolkit()
 {
-	if( _settings->contains( "EnablePolkitSupport" ) ){
-
-		return _settings->value( "EnablePolkitSupport" ).toBool() ;
-	}else{
-		bool e = false ;
-		_settings->setValue( "EnablePolkitSupport",e ) ;
-		return e ;
-	}
-}
-
-bool utility::enablePolkitSupport()
-{
-	return utility::useZuluPolkit() ;
-}
-
-void utility::enablePolkitSupport( bool e )
-{
-	_settings->setValue( "EnablePolkitSupport",e ) ;
+	return _use_polkit ;
 }
 
 void utility::quitHelper()
@@ -323,7 +354,7 @@ QString utility::homePath()
 
 		auto e = opener + " " + utility::Task::makePath( path ) ;
 
-		return utility::Task::run( e,false ).get().failed() ;
+		return utility::Task::run( e ).get().failed() ;
 	} ) ;
 }
 
@@ -336,23 +367,12 @@ void utility::openPath( const QString& path,const QString& opener,
 
 			if( failed && obj ){
 
-				DialogMsg m( obj ) ;
-				m.ShowUIOK( title,msg ) ;
+				DialogMsg( obj ).ShowUIOK( title,msg ) ;
 			}
 		} ) ;
 	}
 }
 
-bool utility::runningInMixedMode()
-{
-	return utility::useZuluPolkit() ;
-}
-
-bool utility::notRunningInMixedMode()
-{
-	return !utility::runningInMixedMode() ;
-}
-
 Task::future< utility::fsInfo >& utility::fileSystemInfo( const QString& q )
 {
 	return ::Task::run< utility::fsInfo >( [ = ](){
@@ -741,115 +761,30 @@ GNU General Public License for more details.\n\
 	DialogMsg( parent,nullptr ).ShowUIInfo( QObject::tr( "about SiriKali" ),true,license ) ;
 }
 
-static utility::array_t _default_dimensions( const char * defaults )
+utility::windowDimensions utility::getWindowDimensions()
 {
-	auto l = QString( defaults ).split( ' ' ) ;
+	QString defaults = "205 149 861 466 326 320 101 76" ;
 
-	utility::array_t e ;
-
-	auto f = e.data() ;
-
-	auto j = l.size() ;
-
-	for( int i = 0 ; i < j ; i++ ){
-
-		*( f + i ) = l.at( i ).toInt() ;
-	}
-
-	return e ;
-}
-
-static utility::array_t _dimensions( const char * defaults,int size )
-{
 	if( _settings->contains( "Dimensions" ) ){
 
-		auto l = utility::split( _settings->value( "Dimensions" ).toString(),' ' ) ;
+		utility::windowDimensions e( _settings->value( "Dimensions" ).toString() ) ;
 
-		utility::array_t p ;
+		if( e ){
 
-		if( l.size() != size || size > int( p.size() ) ){
-
-			utility::debug() << "failed to parse config option" ;
-			return _default_dimensions( defaults ) ;
-		}
-
-		auto f = p.data() ;
-
-		auto j = l.size() ;
-
-		for( int i = 0 ; i < j ; i++ ){
-
-			bool ok ;
-
-			int e = l.at( i ).toInt( &ok ) ;
-
-			if( ok ){
-
-				*( f + i ) = e ;
-			}else{
-				utility::debug() << "failed to parse config option" ;
-				return _default_dimensions( defaults ) ;
-			}
-		}
-
-		return p ;
-	}else{
-		_settings->setValue( "Dimensions",QString( defaults ) ) ;
-		return _default_dimensions( defaults ) ;
-	}
-}
-
-utility::array_t utility::getWindowDimensions()
-{
-	return _dimensions( "205 149 861 466 326 320 101 76",8 ) ;
-}
-
-void utility::setWindowDimensions( const std::initializer_list<int>& e )
-{
-	_settings->setValue( "Dimensions",[ & ](){
-
-		QString q ;
-
-		for( const auto& it : e ){
-
-			q += QString::number( it ) + " " ;
-		}
-
-		return q ;
-	}() ) ;
-}
-
-int utility::pluginKey( QWidget * w,QDialog * d,QByteArray * key,plugins::plugin plugin )
-{
-	QString s ;
-
-	if( plugin == plugins::plugin::hmac_key ){
-
-		s = QObject::tr( "hmac plugin.\n\nThis plugin generates a key using below formular:\n\nkey = hmac(sha256,passphrase,keyfile contents)" ) ;
-
-	}else if( plugin == plugins::plugin::externalExecutable ){
-
-		s = QObject::tr( "This plugin delegates key generation to an external application" ) ;
-	}else{
-		return 1 ;
-	}
-
-	QEventLoop l ;
-
-	plugin::instance( w,d,plugin,[ & ]( const QByteArray& e ){
-
-		*key = e ;
-
-		if( e.isEmpty() ){
-
-			l.exit( 1 ) ;
+			return e ;
 		}else{
-			l.exit( 0 ) ;
+			utility::debug() << "failed to parse config option" ;
+			return defaults ;
 		}
+	}else{
+		_settings->setValue( "Dimensions",defaults ) ;
+		return defaults ;
+	}
+}
 
-	},s ) ;
-
-	return l.exec() ;
+void utility::setWindowDimensions( const utility::windowDimensions& e )
+{
+	_settings->setValue( "Dimensions",e.dimensions() ) ;
 }
 
 template< typename T >
@@ -980,7 +915,14 @@ QString utility::walletName( LXQt::Wallet::BackEnd s )
 {
 	if( s == LXQt::Wallet::BackEnd::kwallet ){
 
-		return "default" ;
+		if( _settings->contains( "KWalletName" ) ){
+
+			return _settings->value( "KWalletName" ).toString() ;
+		}else{
+			QString s = "default" ;
+			_settings->setValue( "KWalletName",s ) ;
+			return s ;
+		}
 	}else{
 		return utility::walletName() ;
 	}
@@ -1275,30 +1217,25 @@ void utility::setWindowOptions( QDialog * w )
 	}
 }
 
-static bool _use_default_widget_relationship()
-{
-	if( _settings->contains( "UseDefaultWidgetRelationship" ) ){
-
-		return _settings->value( "UseDefaultWidgetRelationship" ).toBool() ;
-	}else{
-		bool e = true ;
-		_settings->setValue( "UseDefaultWidgetRelationship",e ) ;
-		return e ;
-	}
-}
-
 void utility::setParent( QWidget * parent,QWidget ** localParent,QDialog * dialog )
 {
-	if( utility::platformIsLinux() ){
+	auto _default_parent = [](){
 
-		if( _use_default_widget_relationship() ){
+		if( _settings->contains( "UseDefaultWidgetRelationship" ) ){
 
-			*localParent = parent ;
+			return _settings->value( "UseDefaultWidgetRelationship" ).toBool() ;
 		}else{
-			*localParent = dialog ;
+			bool e = true ;
+			_settings->setValue( "UseDefaultWidgetRelationship",e ) ;
+			return e ;
 		}
-	}else{
+	}() ;
+
+	if( _default_parent ){
+
 		*localParent = dialog ;
+	}else{
+		*localParent = parent ;
 	}
 }
 
@@ -1414,6 +1351,25 @@ static int _readPasswordMaximumLength()
 	}
 }
 
+bool utility::startMinimized()
+{
+	if( _settings->contains( "StartMinimized" ) ){
+
+		return _settings->value( "StartMinimized" ).toBool() ;
+	}else{
+		bool s = false  ;
+
+		_settings->setValue( "StartMinimized",s ) ;
+
+		return s ;
+	}
+}
+
+void utility::setStartMinimized( bool e )
+{
+	_settings->setValue( "StartMinimized",e ) ;
+}
+
 static inline bool _terminalEchoOff( struct termios * old,struct termios * current )
 {
 	if( tcgetattr( 1,old ) != 0 ){
@@ -1518,3 +1474,83 @@ QProcessEnvironment utility::systemEnvironment()
 
 	return e ;
 }
+
+void utility::windowDimensions::setDimensions( const QStringList& e )
+{
+	m_ok = int( e.size() ) == int( m_array.size() ) ;
+
+	if( m_ok ){
+
+		using tp = decltype( m_array.size() ) ;
+
+		for( tp i = 0 ; i < m_array.size() ; i++ ){
+
+			m_array[ i ] = e.at( i ).toInt( &m_ok ) ;
+
+			if( !m_ok ){
+
+				break ;
+			}
+		}
+	}else{
+		utility::debug() << "window dimensions do not match data structure size" ;
+	}
+}
+
+utility::windowDimensions::windowDimensions( const QStringList& e )
+{
+	this->setDimensions( e ) ;
+}
+
+utility::windowDimensions::windowDimensions( const QString& e )
+{
+	this->setDimensions( utility::split( e,' ' ) ) ;
+}
+
+utility::windowDimensions::windowDimensions( const std::array< int,size >& e )
+	: m_array( e ),m_ok( true )
+{
+}
+
+utility::windowDimensions::operator bool()
+{
+	return m_ok ;
+}
+
+int utility::windowDimensions::columnWidthAt( std::array< int,size >::size_type s ) const
+{
+	auto e = s + 4 ;
+
+	if( e < m_array.size() ){
+
+		return m_array[ e ] ;
+	}else{
+		utility::debug() << "window dimension index out of range" ;
+		return 0 ;
+	}
+}
+
+QRect utility::windowDimensions::geometry() const
+{
+	auto e = m_array.data() ;
+
+	return { *( e + 0 ),*( e + 1 ),*( e + 2 ),*( e + 3 ) } ;
+}
+
+QString utility::windowDimensions::dimensions() const
+{
+	auto _number = []( const int * s,size_t n ){ return QString::number( *( s + n ) ) ; } ;
+
+	auto s = m_array.data() ;
+
+	auto e = _number( s,0 ) ;
+
+	using tp = decltype( m_array.size() ) ;
+
+	for( tp i = 1 ; i < m_array.size() ; i++ ){
+
+		e += " " + _number( s,i ) ;
+	}
+
+	return e ;
+}
