@@ -38,6 +38,8 @@
 #include <QAction>
 #include <QIcon>
 #include <QSettings>
+#include <QRect>
+
 #include <functional>
 #include <memory>
 #include <array>
@@ -229,9 +231,6 @@ namespace utility
 	bool reUseMountPoint( void ) ;
 	void reUseMountPoint( bool ) ;
 
-	bool enablePolkitSupport( void ) ;
-	void enablePolkitSupport( bool ) ;
-
 	QString homeConfigPath( const QString& = QString() ) ;
 	QString homePath() ;
 	QString mountPath( const QString& path ) ;
@@ -274,12 +273,15 @@ namespace utility
 	bool autoMountFavoritesOnAvailable() ;
 	void autoMountFavoritesOnAvailable( bool ) ;
 
+	bool startMinimized() ;
+	void setStartMinimized( bool ) ;
+
 	int checkForUpdateInterval( void ) ;
 
 	bool enableRevealingPasswords( void ) ;
 
-	bool runningInMixedMode() ;
-	bool notRunningInMixedMode() ;
+	enum class background_thread{ True,False } ;
+	bool enablePolkit( utility::background_thread ) ;
 
 	QProcessEnvironment systemEnvironment() ;
 
@@ -290,8 +292,7 @@ namespace utility
 	QStringList executableSearchPaths( void ) ;
 	QString executableSearchPaths( const QString& ) ;
 
-	bool useZuluPolkit( void ) ;
-	void startHelperExecutable( QWidget *,const QString&,const char *,const char * ) ;
+	bool useSiriPolkit( void ) ;
 	void quitHelper() ;
 	QString helperSocketPath() ;
 	void clearFavorites( void ) ;
@@ -315,12 +316,24 @@ namespace utility
 	void setLocalizationLanguage( bool translate,QMenu * m,utility2::translator& ) ;
 	void languageMenu( QMenu *,QAction *,utility2::translator& ) ;
 
-	using array_t = std::array< int,8 > ;
+	class windowDimensions{
+	public:
+		static constexpr int size = 8 ;
+		windowDimensions( const QStringList& e ) ;
+		windowDimensions( const QString& e ) ;
+		windowDimensions( const std::array< int,size >& e ) ;
+		operator bool() ;
+		int columnWidthAt( std::array< int,size >::size_type ) const ;
+		QRect geometry() const ;
+		QString dimensions() const ;
+	private:
+		void setDimensions( const QStringList& ) ;
+		std::array< int,size > m_array ;
+		bool m_ok = false ;
+	};
 
-	utility::array_t getWindowDimensions() ;
-	void setWindowDimensions( const std::initializer_list<int>& ) ;
-
-	int pluginKey( QWidget *,QDialog *,QByteArray *,plugins::plugin ) ;
+	utility::windowDimensions getWindowDimensions() ;
+	void setWindowDimensions( const utility::windowDimensions& ) ;
 
 	::Task::future< bool >& openPath( const QString& path,const QString& opener ) ;
 
@@ -329,6 +342,14 @@ namespace utility
 
 namespace utility
 {
+	template< typename T >
+	QByteArray fileContents( const T& e )
+	{
+		QFile file( e ) ;
+		file.open( QIODevice::ReadOnly ) ;
+		return file.readAll() ;
+	}
+
 	bool pathExists( const QString& ) ;
 
 	template< typename ... F >
@@ -411,15 +432,20 @@ namespace utility
 
 		static ::Task::future< utility::Task >& run( const QString& exe,int,bool e ) ;
 
-		static ::Task::future< utility::Task >& run( const QString& exe )
+		static ::Task::future< utility::Task >& run( const QString& exe,
+							     const QByteArray& password = QByteArray() )
 		{
-			return ::Task::run< utility::Task >( [ exe ](){ return utility::Task( exe ) ; } ) ;
+			return ::Task::run< utility::Task >( [ = ](){
+
+				return utility::Task( exe,-1,utility::systemEnvironment(),password ) ;
+			} ) ;
 		}
+
 		static void exec( const QString& exe,
 				  const QProcessEnvironment& env = utility::systemEnvironment(),
 				  std::function< void() > f = [](){} )
 		{
-			::Task::run< utility::Task >( [ = ](){ return utility::Task( exe,env,f ) ; } ).start() ;
+			::Task::exec( [ = ](){ utility::Task( exe,env,f ) ; } ) ;
 		}
 		static void wait( int s )
 		{
@@ -458,12 +484,12 @@ namespace utility
 		Task()
 		{
 		}
-		Task( const QString& exe,int waitTime = -1,const QProcessEnvironment& env = QProcessEnvironment(),
-		      const QByteArray& password = QByteArray(),std::function< void() > f = [](){},bool e = true )
+		Task( const QString& exe,int waitTime = -1,const QProcessEnvironment& env = utility::systemEnvironment(),
+		      const QByteArray& password = QByteArray(),std::function< void() > f = [](){},bool e = false )
 		{
 			this->execute( exe,waitTime,env,password,std::move( f ),e ) ;
 		}
-		Task( const QString& exe,const QProcessEnvironment& env,std::function< void() > f,bool e = true )
+		Task( const QString& exe,const QProcessEnvironment& env,std::function< void() > f,bool e = false )
 		{
 			this->execute( exe,-1,env,QByteArray(),std::move( f ),e ) ;
 		}
